# Written by John Hoffman
# see LICENSE.txt for license information
# Updated and modified for ABC_OKC : Old King Cole

from array import array
from time import clock


class SingleBuffer:
    # [:] returns self.buf and not a copy of self.buf ; do not release such a buffer
    # before ending all treatments on self.buf ;
    # Use getArrayCopy to get a copy
    def __init__(self, pool, piecebuf):
        self.bufferpool = pool
        self.piecebuf = piecebuf
        self.buf = array('c')
        self.lastrelease = 0

    def init(self):
        self.length = 0

    def append(self, s):
        l = self.length + len(s)
        self.buf[self.length:l] = array('c', s)
        self.length = l

    def __len__(self):
        return self.length

    def __getslice__(self, a, b):
        if b > self.length:
            b = self.length
        elif b < 0:
            b += self.length
        if a == 0 and len(self.buf) == b == self.length:
            return self.buf  # optimization
        return self.buf[a:b]

    def getArrayCopy(self):
        return self.buf[:self.length]

    def release(self):
        self.lastrelease = clock()
        if not self.piecebuf:
            del self.buf[self.length:]
        self.bufferpool.release(self)


class BufferPool:
    def __init__(self, sched, config):
        self.piecepool = []
        self.chunkpool = []
        self.sched = sched
        self.read_buffer_pool_expiration = config['read_buffer_pool_expiration']
        self.sched(self.expireBuffer, 5)

    def new(self, piecebuf):
        if piecebuf:
            pool = self.piecepool
        else:
            pool = self.chunkpool
        if pool:
            x = pool.pop()
        else:
            x = SingleBuffer(self, piecebuf)
        x.init()
        return x

    def release(self, x):
        if x.piecebuf:
            self.piecepool.append(x)
        else:
            self.chunkpool.append(x)

    def clear(self):
        del self.piecepool[:]
        del self.chunkpool[:]

    def expireBuffer(self):
        now = clock()
        for buffer in self.piecepool[:]:
            if now - buffer.lastrelease > self.read_buffer_pool_expiration:
                self.piecepool.remove(buffer)
        for buffer in self.chunkpool[:]:
            if now - buffer.lastrelease > self.read_buffer_pool_expiration:
                self.chunkpool.remove(buffer)
        self.sched(self.expireBuffer, 5)
